<?php

declare(strict_types=1);
require_once __DIR__.'/../../../admin/includes/admininclude.php';
$oAccount->permission('PLUGIN_ADMIN_VIEW', true, true);

require \dirname(__DIR__, 1).'/src/warmup.php';

$params = array_merge_recursive($_GET, $_POST);

require __DIR__.'/../src/warmup.php';

function process_csv($file)
{
  global $P;
  $dbh = $P->getPDO();
  $dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

  $dbh->query('DROP TABLE IF EXISTS `xplugin_cin_kbafinder`');

  $dbh->query("CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder` (
    `Hersteller` VARCHAR(60) NOT NULL COLLATE 'utf8_general_ci',
    `Modell` VARCHAR(60) NOT NULL COLLATE 'utf8_general_ci',
    `Typ` VARCHAR(100) DEFAULT NULL COLLATE 'utf8_general_ci',
    `ccmTech` VARCHAR(5) DEFAULT NULL,
    `PS` VARCHAR(4) DEFAULT NULL,
    `KW` VARCHAR(4) DEFAULT NULL,
    `BjVon` VARCHAR(7) DEFAULT NULL,
    `BjBis` VARCHAR(7) DEFAULT NULL,
    `KBANr` VARCHAR(7) NOT NULL,
    `carId` VARCHAR(9) DEFAULT NULL,
    INDEX `Hersteller` (`Hersteller`),
    INDEX `Modell` (`Modell`),
    INDEX `Hersteller_Modell` (`Hersteller`, `Modell`),
    INDEX `KBANr` (`KBANr`),
    INDEX `carId` (`carId`)
  )
  COLLATE='utf8_general_ci'
  ENGINE=InnoDB");

  $columns = array_filter(json_decode(file_get_contents(__DIR__.'/../common/columns.json'), true), static function ($v): bool {
    return $v;
  }, \ARRAY_FILTER_USE_BOTH);

  $columnNames = [
    'manufacturer' => 'Hersteller',
    'model' => 'Modell',
    'type' => 'Typ',
    'displacement' => 'ccmTech',
    'hp' => 'PS',
    'kw' => 'KW',
    'from' => 'BjVon',
    'to' => 'BjBis',
    'kbanumber' => 'KBANr',
  ];

  $ziparchive = new ZipArchive();
  $zipstatus = $ziparchive->open($file);
  if (true !== $zipstatus) {
    throw (new Exception('Es gibt ein Problem mit der Zip-Datei'));

    return;
  }

  $sql = 'INSERT INTO xplugin_cin_kbafinder ([NAMES]) VALUES ([VALUES])';

  $names = implode(', ', array_map(static function ($column) use ($columnNames): string {
    return $columnNames[$column];
  }, array_keys($columns)));

  $values = implode(', ', array_fill(0, \count($columns), '?'));
  $sql = str_replace('[NAMES]', $names, $sql);
  $sql = str_replace('[VALUES]', $values, $sql);

  $dbh->query('TRUNCATE TABLE xplugin_cin_kbafinder');
  $sth = $dbh->prepare($sql);

  $stream = $ziparchive->getStream($ziparchive->getNameIndex(0));
  $inserted = 0;
  $missed = 0;
  $dbh->query('ALTER TABLE `xplugin_cin_kbafinder` DISABLE KEYS');
  $dbh->beginTransaction();
  $currentLine = 0;
  $missedLines = [];
  while ( ! feof($stream)) {
    ++$currentLine;
    $line = fgets($stream);
    $columns = str_getcsv($line);
    if ($sth->execute($columns)) {
      ++$inserted;
    } else {
      $missedLines[] = $currentLine;
      ++$missed;
    }
  }
  $dbh->commit();
  $dbh->query('ALTER TABLE `xplugin_cin_kbafinder` ENABLE KEYS');

  fclose($stream);

  return [
    'inserted' => $inserted,
    'missed' => $missed,
    'missedLines' => $missedLines,
  ];
}

$response = ['data' => null, 'messages' => [], 'errors' => []];
$result = null;
switch ($params['action']) {
  case 'upload':
    $archiveFile = $_FILES[0];

    try {
      $result = process_csv($archiveFile['tmp_name']);
    } catch (Exception $e) {
      $response['errors'][] = $e->getMessage();
    }
    if (file_exists(__DIR__.'/../frontend/storedChecksum.txt')) {
      unlink(__DIR__.'/../frontend/storedChecksum.txt');
    }
    $update = date('d.m.Y H:i:s');
    $of = fopen('lastUpdate.txt', 'wb');
    fwrite($of, $update);
    fclose($of);
    $response['data'] = ['lastUpdate' => $update, 'result' => $result];

    break;
}
header('Content-Type: application/json');
echo json_encode($response);
