<?php

declare(strict_types=1);

require \dirname(__DIR__, 1).'/src/warmup.php';

$seconds_to_cache = 3600;
$ts = gmdate('D, d M Y H:i:s', time() + $seconds_to_cache).' GMT';
header("Expires: {$ts}");
header('Pragma: cache');
header("Cache-Control: max-age={$seconds_to_cache}");
header('Content-Type: application/json');

if ( ! $L->canRun()) {
  echo '[]';

  return;
}

if (isset($_COOKIE['PLUGINDEBUG']) || \PHP_SAPI === 'cli') {
  require_once \dirname(__DIR__, 1).'/src/dev_errors.php';
}

date_default_timezone_set('Europe/Berlin');

ini_set('max_execution_time', '0');

if ( ! file_exists(__DIR__.'/kbatree.json') || filemtime(__DIR__.'/kbatree.json') < time() - 3600) {
  $lock = fopen(__DIR__.'/lock', 'cb');
  if (flock($lock, \LOCK_EX | \LOCK_NB)) {
    $P->getPDO()->query('DROP TABLE IF EXISTS xplugin_cin_kbafinder_listed_kbas');
    $P->getPDO()->query("CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder_listed_kbas` (
      `id` int(11) UNSIGNED AUTO_INCREMENT PRIMARY KEY,
      `kbanr` varchar(7)
    ) COLLATE='utf8_general_ci'
    ENGINE=InnoDB");
    $P->getPDO()->query('insert into xplugin_cin_kbafinder_listed_kbas(kbanr) select distinct kbanr from xplugin_cin_kbafinder_helper');

    $sql = "SELECT TRIM(Hersteller) AS Hersteller,
    TRIM(Modell) AS Modell,
    TRIM(Typ) AS Typ,
    TRIM(TRAILING '.' FROM CONCAT(TRIM(BjVon), ' - ', TRIM(BjBis))) AS Baujahr,
    CONCAT(TRIM(KW), '/', TRIM(PS)) AS Leistung,
    TRIM(ccmTech) AS Hubraum,
    xplugin_cin_kbafinder.KBANr,
    xplugin_cin_kbafinder.carId
    FROM xplugin_cin_kbafinder
    LEFT JOIN xplugin_cin_kbafinder_listed_kbas ON xplugin_cin_kbafinder.kbanr = xplugin_cin_kbafinder_listed_kbas.kbanr
    WHERE xplugin_cin_kbafinder_listed_kbas.id IS NOT NULL
    ORDER BY Hersteller, Modell, Typ";
    $statement = $P->getPDO()->prepare($sql);
    $statement->execute();

    $kbatree = new stdClass();

    while ($row = $statement->fetch(PDO::FETCH_ASSOC)) {
      if ( ! isset($kbatree->{$row['Hersteller']})) {
        $kbatree->{$row['Hersteller']} = new stdClass();
      }
      if ( ! isset($kbatree->{$row['Hersteller']}->{$row['Modell']})) {
        $kbatree->{$row['Hersteller']}->{$row['Modell']} = new stdClass();
      }
      if ( ! isset($kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']})) {
        $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']} = new stdClass();
      }
      if ( ! isset($kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']})) {
        $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']} = new stdClass();
      }
      $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']}->BJ = $row['Baujahr'];
      $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']}->KWPS = $row['Leistung'];
      $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']}->CCM = $row['Hubraum'];
      $kbatree->{$row['Hersteller']}->{$row['Modell']}->{$row['Typ']}->{$row['KBANr']}->KBANr = $row['KBANr'];
    }
    $kbajson = json_encode((array) $kbatree, \JSON_UNESCAPED_SLASHES | \JSON_UNESCAPED_UNICODE);
    file_put_contents(__DIR__.'/kbatree.json', $kbajson);
    fclose($lock);
    unlink(__DIR__.'/lock');
  }
}

$kbajson = file_get_contents(__DIR__.'/kbatree.json');
echo '[]' === $kbajson ? '{}' : $kbajson;
