<?php

declare(strict_types=1);

function byteConvert($bytes)
{
  if (0 === $bytes) {
    return '0.00 B';
  }

  $s = ['B', 'KB', 'MB', 'GB', 'TB', 'PB'];
  $e = floor(log($bytes, 1024));

  return round($bytes / 1024 ** $e, 2).$s[$e];
}

out('tecdoccrawl: starting');

$providerIdFile = __DIR__.'/../providerid';
$providerid = null;
$apikeyFile = __DIR__.'/../apikey';
$apikey = null;
$articleCountryFile = __DIR__.'/../articlecountry';
$articlecountry = null;

if (file_exists($providerIdFile)) {
  $providerid = trim(file_get_contents($providerIdFile));
}
if (file_exists($apikeyFile)) {
  $apikey = trim(file_get_contents($apikeyFile));
}
if (file_exists($articleCountryFile)) {
  $articlecountry = trim(file_get_contents($articleCountryFile));
}

if (null === $providerid || '' === $providerid
  || null === $apikey || '' === $apikey
  || null === $articlecountry || '' === $articlecountry) {
  out('tecdoccrawl: credentials missing, quitting');

  return;
}

function formatDate($date)
{
  if ('' === $date) {
    return '';
  }

  return mb_substr((string) $date, 0, 4).'.'.mb_substr((string) $date, 4);
}

$crawler = new Cin\KBAFinder5\TecDocCrawl($providerid, $articlecountry, $apikey);
$list = $crawler->crawl();

if ($list) {
  ini_set('max_execution_time', '0');
  out('tecdoccrawl: updating database from fresh list');

  $P->getPDO()->query('DROP TABLE IF EXISTS `xplugin_cin_kbafinder`');
  $P->getPDO()->query("CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder` (
    `Hersteller` VARCHAR(60) NOT NULL COLLATE 'utf8_general_ci',
    `Modell` VARCHAR(60) NOT NULL COLLATE 'utf8_general_ci',
    `Typ` VARCHAR(60) NOT NULL COLLATE 'utf8_general_ci',
    `ccmTech` VARCHAR(5) NOT NULL,
    `PS` VARCHAR(4) NOT NULL,
    `KW` VARCHAR(4) NOT NULL,
    `BjVon` VARCHAR(7) NOT NULL,
    `BjBis` VARCHAR(7) NOT NULL,
    `KBANr` VARCHAR(7) NOT NULL,
    `carId` VARCHAR(9) DEFAULT NULL,
    INDEX `Hersteller` (`Hersteller`),
    INDEX `Modell` (`Modell`),
    INDEX `Hersteller_Modell` (`Hersteller`, `Modell`),
    INDEX `KBANr` (`KBANr`),
    INDEX `carId` (`carId`)
  )
  COLLATE='utf8_general_ci'
  ENGINE=InnoDB");

  $P->getPDO()->query('SET sql_mode=""');

  $insert = $P->getPDO()->prepare('INSERT INTO `xplugin_cin_kbafinder` (`Hersteller`, `Modell`, `Typ`, `ccmTech`, `PS`, `KW`, `BjVon`, `BjBis`, `KBANr`, `carId`) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)');

  $P->getPDO()->query('ALTER TABLE `xplugin_cin_kbafinder` DISABLE KEYS');

  $P->getPDO()->beginTransaction();
  foreach ($list as $v) {
    foreach ($v[8] as $kba) {
      $values = $v;
      $values[6] = formatDate($v[6]);
      $values[7] = formatDate($v[7]);
      $values[8] = $kba;
      $insert->execute($values);
    }
  }
  $P->getPDO()->commit();

  $P->getPDO()->query('ALTER TABLE `xplugin_cin_kbafinder` ENABLE KEYS');

  out('tecdoccrawl: memory usage '.byteConvert(memory_get_peak_usage(true)));
} else {
  out('tecdoccrawl: memory usage '.byteConvert(memory_get_peak_usage(true)));
  out('tecdoccrawl: nothing to do yet');
}
