<?php

declare(strict_types=1);

$documentsPerRun = 500;
$indexStart = time();

out('normalize kbanrs: starting');

// alternatively create a partitioned table
// Create Table | CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder_helper` (
//   `kartikel` int(11) DEFAULT NULL,
//   `kbanr` varchar(7) DEFAULT NULL,
//   `pidx` varchar(2) NOT NULL,
//   KEY `kartikel` (`kartikel`),
//   KEY `kbanr` (`kbanr`),
//   KEY `kartikel_kbanr` (`kartikel`,`kbanr`),
//   KEY `pidx` (`pidx`) USING BTREE
// ) ENGINE=InnoDB DEFAULT CHARSET=utf8
//  PARTITION BY KEY (`pidx`)
// PARTITIONS 51

$P->getPDO()->query("CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder_helper` (
  `kartikel` INT(11) NULL DEFAULT NULL,
  `kbanr` VARCHAR(7) NULL DEFAULT NULL,
  KEY `kartikel` (`kartikel`),
  KEY `kbanr` (`kbanr`),
  KEY `kartikel_kbanr` (`kartikel`,`kbanr`)
)
COLLATE='utf8_general_ci'
ENGINE=InnoDB");

$laststale = 0;
if (file_exists(__DIR__.'/laststale')) {
  $laststale = filemtime(__DIR__.'/laststale');
}

if ( ! $partitioned && $laststale < time() - 86415) {
  touch(__DIR__.'/laststale');
  out('normalize kbanrs: cleaning normalized table from stale entries');
  $delStaleStmt = $P->getPDO()->prepare('DELETE FROM xplugin_cin_kbafinder_helper
  WHERE NOT EXISTS(SELECT NULL FROM tartikel t WHERE t.kArtikel = xplugin_cin_kbafinder_helper.kartikel)');
  $delStaleStmt->execute();
}

$companionInstalled = 1 === (int) $P->getPDO()->query("select count(*) cnt from tplugin t where t.cPluginID = 'cin_crossingparts_5' and t.nStatus = 2")->fetchColumn();
if ($companionInstalled) {
  out('normalize kbanrs: companion installed. let her do the heavy lifting');

  return;
}

$P->getPDO()->query("CREATE TABLE IF NOT EXISTS `xplugin_cin_kbafinder_article_tracker` (
  `kartikel` INT(11) NOT NULL
)
COLLATE='utf8_general_ci'
ENGINE=InnoDB");

$latestDocumentFileName = __DIR__.'/latestDocumentDate';

if (file_exists($latestDocumentFileName) && filesize($latestDocumentFileName) > 0) {
  $latestDocumentDate = file_get_contents($latestDocumentFileName);
} else {
  $latestDocumentDate = $P->getPDO()
    ->query('SELECT MIN(dLetzteAktualisierung) minDate FROM tartikel')
    ->fetchColumn();
}

out('normalize kbanrs: fetching articles changed since: '.$latestDocumentDate);
$fetchStmt = $P->getPDO()->prepare("SELECT t.kArtikel, t2.cWert kbanrs, t.dLetzteAktualisierung touched FROM tartikel t
  LEFT JOIN tartikelattribut t2 ON t.kArtikel = t2.kArtikel AND t2.cName = 'KBANrn'
  WHERE t.dLetzteAktualisierung >= ?
    AND t2.cWert IS NOT NULL
    AND NOT EXISTS (SELECT 1 FROM `xplugin_cin_kbafinder_article_tracker` tr WHERE tr.kartikel = t.kArtikel)
  ORDER BY t.dLetzteAktualisierung ASC
  LIMIT ".$documentsPerRun);
$fetchStmt->execute([$latestDocumentDate]);
$articles = $fetchStmt->fetchAll(PDO::FETCH_ASSOC);

$kartikelArr = array_map(static function ($article) {
  return $article['kArtikel'];
}, $articles);

$articleCount = \count($articles);
if ($articleCount > 0) {
  out('normalize kbanrs: cleaning kArtikel ('.implode(', ', $kartikelArr).')');
  $cleaningStmt = $P->getPDO()->prepare('DELETE FROM xplugin_cin_kbafinder_helper WHERE kartikel IN ('.str_repeat('?,', \count($kartikelArr) - 1).'?'.')');
  $cleaningStmt->execute($kartikelArr);

  if ($partitioned) {
    $kbaSQL = [];
    foreach ($articles as $article) {
      foreach (explode(',', $article['kbanrs']) as $kba) {
        $pidx = mb_substr($kba, 0, 2);
        $kbaSQL[] = "('{$article['kArtikel']}', '{$kba}', '{$pidx}')";
      }
    }
    $kbaSQL = implode(",\n", $kbaSQL);
    out('normalize kbanrs: inserting kbanrs (partitioned)');
    $P->getPDO()->query("INSERT INTO xplugin_cin_kbafinder_helper (kartikel, kbanr, pidx) VALUES {$kbaSQL}");
  } else {
    $kbaSQL = [];
    foreach ($articles as $article) {
      foreach (explode(',', $article['kbanrs']) as $kba) {
        $kbaSQL[] = "('{$article['kArtikel']}', '{$kba}')";
      }
    }
    $kbaSQL = implode(",\n", $kbaSQL);
    out('normalize kbanrs: inserting kbanrs');
    $P->getPDO()->query("INSERT INTO xplugin_cin_kbafinder_helper (kartikel, kbanr) VALUES {$kbaSQL}");
  }
}

$indexEnd = time() - $indexStart;
if ($articleCount === $documentsPerRun) {
  if ($articles[$articleCount - 1]['touched'] === $latestDocumentDate) {
    $kArtikels = implode(',', array_map(static function ($article) {
      return '('.$article['kArtikel'].')';
    }, $articles));
    $P->getPDO()->query('INSERT INTO `xplugin_cin_kbafinder_article_tracker` (kArtikel) VALUES '.$kArtikels);
  } else {
    $P->getPDO()->query('TRUNCATE TABLE `xplugin_cin_kbafinder_article_tracker`');
    $lastDocumentDate = $articles[$articleCount - 1]['touched'];
    $sameDateTime = array_values(array_filter($articles, static function ($article) use ($lastDocumentDate) {
      return $article['touched'] === $lastDocumentDate;
    }));
    $kArtikels = implode(',', array_map(static function ($article) {
      return '('.$article['kArtikel'].')';
    }, $sameDateTime));
    $P->getPDO()->query('INSERT INTO `xplugin_cin_kbafinder_article_tracker` (kArtikel) VALUES '.$kArtikels);
  }
  file_put_contents($latestDocumentFileName, $articles[$articleCount - 1]['touched']);
  out("normalize kbanrs: normalizing {$articleCount} articles took {$indexEnd} seconds");
} elseif ($articleCount > 0) {
  $latestDocumentDate = date('Y-m-d H:i:s', strtotime($articles[$articleCount - 1]['touched']) + 1);
  file_put_contents($latestDocumentFileName, $latestDocumentDate);
  $P->getPDO()->query('TRUNCATE TABLE `xplugin_cin_kbafinder_article_tracker`');
  out("normalize kbanrs: normalizing {$articleCount} articles took {$indexEnd} seconds");
} else {
  out('normalize kbanrs: index complete');
}
out('normalize kbanrs: done');
